## -*- coding: utf-8 -*-

import listener
import univention.uldap
from univention.ox.oxlogger import OXLogger
from univention.ox.oxlistener import ContinueHookCommand, ExitHookCommand
from univention.ox.listener_tools import AsRoot, get_credentials,  ox_call, listener_is_deactivated
from ldap.filter import filter_format

logger = OXLogger('ox-user')

capabilities=[
	'oxCapability-ez-admin',
	'oxCapability-ez-core',
	'oxCapability-ez-sales',
	'oxCapability-ez-project']


ldapbase = listener.configRegistry['ldap/base']

def get_context_credentials(contextid):
        admuser = 'oxadmin-context%s' % (contextid,)
        if admuser == 'oxadmin-context10':
                admuser = 'oxadmin'
        admpass = get_credentials('/etc/ox-secrets/context%s.secret' % (contextid,))
        return admuser, admpass

def get_default_context_id():
        return listener.configRegistry.get('ox/context/id', '10')


def get_context_id_from_domain(ldapCon, domainname):
        filter_s = filter_format('(&(objectClass=oxMailDomainServerSettings)(cn=%s))', (domainname,))
        query_result = ldapCon.search(base='cn=domain,cn=mail,%s' % ldapbase, filter=filter_s, attr=['oxContextName'])
        if not query_result:
                return ''
        context_name = query_result[0][1].values()[0][0]

        filter_s = filter_format('(&(objectClass=oxContext)(cn=%s))', (context_name,))
        query_result = ldapCon.search(base='cn=open-xchange,%s' % ldapbase, filter=filter_s, attr=['oxContextIDNum'])
        if not query_result:
                return ''
        return query_result[0][1].values()[0][0]


def get_context_id_from_user(old_or_new):
        return old_or_new.get('oxContextIDNum', [''])[0]



def user_post_handler_hook(dn, new, old, command, *args, **kwargs):
        contextnum = ''
        oxhomeserver = ''
        admuser = ''
        admpass = ''
        groupwareoptions = []
        email = '@'
        if new and new.get('mailPrimaryAddress'):
                        email = new['mailPrimaryAddress'][0]

        if old and old.get('mailPrimaryAddress'):
                        oldemail = old['mailPrimaryAddress'][0]

        if new:
                domain = email.split('@')[1]
        else:
                domain = oldemail.split('@')[1]
        olddomain = oldemail.split('@')[1]

        with AsRoot():
                try:
                        ldapCon = univention.uldap.getMachineConnection(ldap_master=False)
                except IOError as ex:
                        raise ListenerTools_RuntimeError(ex)

        if new:
                contextnum = get_context_id_from_user(new)
                if not contextnum:
                        logger.debug('No context in NEW, using domain "{}" to find context...'.format(domain
))
                        contextnum = get_context_id_from_domain(ldapCon, domain)
                admuser, admpass = get_context_credentials(contextnum)
        authArgs = [
                "--adminuser=%s" % admuser,
                "--adminpass=%s" % admpass,
                "--contextid=%s" % contextnum,
        ]

        blacklisted = ['Administrator', 'join-backup', 'join-slave', 'spam']

        username = ''
        if old.get('uid'):
                username = old['uid'][0]
                logger.process('old username: {}'.format(username))
        if new.get('uid'):
                username2 = new['uid'][0]
                logger.process('new username: {}'.format(username2))
                if username2 != username:
                        if username2 in blacklisted or username in blacklisted:
                                logger.warn('not allowed to change users in blacklist ({})'.format(username))
                                return ContinueHookCommand()
                username = username2
        logger.qualifier = username
	capAdd = []
	capRemove = []
	for cap in capabilities:
		if cap in old:
			oldVal=old[cap][0]
		else:
			oldVal=0
		if cap in new:
			newVal=new[cap][0]
		else:
			newVal=0
		if oldVal!=newVal:
			logger.process('DN : %s cap %s cmd %s val %s' % ( dn, cap, command, newVal))
                        cCap=cap.replace('oxCapability-','')
			if newVal=='1':
				capAdd.append(cCap)
			else:
				capRemove.append(cCap)
			if (cCap == 'ez-core'):
				if newVal=='1':
					capAdd.append("ez-connect")
				else:
					capRemove.append("ez-connect")
	if len(capAdd)>0:
		groupwareoptions.append("--capabilities-to-add=%s" % (",".join(capAdd)))

	if len(capRemove)>0:
		groupwareoptions.append("--capabilities-to-remove=%s" % (",".join(capRemove)))

	if len(groupwareoptions)>0:
		try:
			listener.setuid(0)   # falls benötigt
			cmd = [
				"changeuser",
				"--username=%s" % username,
				] + authArgs + groupwareoptions
			cmdStr=" ".join(cmd)
			logger.process('CMD %s' % cmdStr)
			logger.info('DIGITEC eZ LDAP Modifying OX settings for uid "{}" in context {}'.format(username, contextnum))
			ox_call(cmd, logger=logger)
		finally:
			listener.unsetuid()
	else:
		logger.info('DIGITEC eZ LDAP no modifications"{}" in context {}'.format(username, contextnum))
	return ContinueHookCommand()


def register_hooks():
	return [
		('post_handler_hook', user_post_handler_hook),
	]

